@@ -37,6 +37,11 @@ gboolean udisks_daemon_util_lvm2_name_is_reserved (const gchar *name);
 
 void udisks_daemon_util_lvm2_trigger_udev (const gchar *device_file);
 
+GStrv udisks_daemon_util_lvm2_gather_pvs (UDisksDaemon             *daemon,
+                                          UDisksLinuxVolumeGroupObject  *vgroup_object,
+                                          const gchar *const       *objects_paths,
+                                          GError                  **error);
+
 G_END_DECLS
 
 #endif /* __UDISKS_LVM2_DAEMON_UTIL_H__ */
