@@ -40,9 +40,7 @@ GDBusConnection          *udisks_daemon_get_connection        (UDisksDaemon    *
 GDBusObjectManagerServer *udisks_daemon_get_object_manager    (UDisksDaemon    *daemon);
 UDisksMountMonitor       *udisks_daemon_get_mount_monitor     (UDisksDaemon    *daemon);
 UDisksCrypttabMonitor    *udisks_daemon_get_crypttab_monitor  (UDisksDaemon    *daemon);
-#ifdef HAVE_LIBMOUNT_UTAB
 UDisksUtabMonitor        *udisks_daemon_get_utab_monitor      (UDisksDaemon    *daemon);
-#endif
 UDisksLinuxProvider      *udisks_daemon_get_linux_provider    (UDisksDaemon    *daemon);
 PolkitAuthority          *udisks_daemon_get_authority         (UDisksDaemon    *daemon);
 UDisksState              *udisks_daemon_get_state             (UDisksDaemon    *daemon);
@@ -55,7 +53,7 @@ gboolean                  udisks_daemon_get_enable_tcrypt     (UDisksDaemon    *
 const gchar              *udisks_daemon_get_uuid              (UDisksDaemon    *daemon);
 
 /**
- * UDisksDaemonWaitFunc:
+ * UDisksDaemonWaitFuncGeneric:
  * @daemon: A #UDisksDaemon.
  * @user_data: The #gpointer passed to udisks_daemon_wait_for_object_sync().
  *
