@@ -269,33 +269,17 @@ udisks_linux_mdraid_update (UDisksLinuxMDRaid       *mdraid,
     {
       device = UDISKS_LINUX_DEVICE (member_devices->data);
       num_devices = g_udev_device_get_property_as_int (device->udev_device, "UDISKS_MD_MEMBER_DEVICES");
-      if (! num_devices)
-          num_devices = g_udev_device_get_property_as_int (device->udev_device, "STORAGED_MD_MEMBER_DEVICES");
       level = g_udev_device_get_property (device->udev_device, "UDISKS_MD_MEMBER_LEVEL");
-      if (! level)
-          level = g_udev_device_get_property (device->udev_device, "STORAGED_MD_MEMBER_LEVEL");
       uuid = g_udev_device_get_property (device->udev_device, "UDISKS_MD_MEMBER_UUID");
-      if (! uuid)
-          uuid = g_udev_device_get_property (device->udev_device, "STORAGED_MD_MEMBER_UUID");
       name = g_udev_device_get_property (device->udev_device, "UDISKS_MD_MEMBER_NAME");
-      if (! name)
-          name = g_udev_device_get_property (device->udev_device, "STORAGED_MD_MEMBER_NAME");
     }
   else
     {
       device = UDISKS_LINUX_DEVICE (raid_device);
       num_devices = g_udev_device_get_property_as_int (device->udev_device, "UDISKS_MD_DEVICES");
-      if (! num_devices)
-          num_devices = g_udev_device_get_property_as_int (device->udev_device, "STORAGED_MD_DEVICES");
       level = g_udev_device_get_property (device->udev_device, "UDISKS_MD_LEVEL");
-      if (! level)
-          level = g_udev_device_get_property (device->udev_device, "STORAGED_MD_LEVEL");
       uuid = g_udev_device_get_property (device->udev_device, "UDISKS_MD_UUID");
-      if (! uuid)
-          uuid = g_udev_device_get_property (device->udev_device, "STORAGED_MD_UUID");
       name = g_udev_device_get_property (device->udev_device, "UDISKS_MD_NAME");
-      if (! name)
-          name = g_udev_device_get_property (device->udev_device, "STORAGED_MD_NAME");
     }
 
   /* figure out size */
@@ -329,15 +313,15 @@ udisks_linux_mdraid_update (UDisksLinuxMDRaid       *mdraid,
       if (mdraid_has_redundancy (level))
         {
           /* Can't use GUdevDevice methods as they cache the result and these variables vary */
-          degraded = read_sysfs_attr_as_int (raid_device->udev_device, "md/degraded");
-          sync_action = read_sysfs_attr (raid_device->udev_device, "md/sync_action");
-          sync_completed = read_sysfs_attr (raid_device->udev_device, "md/sync_completed");
-          bitmap_location = read_sysfs_attr (raid_device->udev_device, "md/bitmap/location");
+          degraded = udisks_linux_device_read_sysfs_attr_as_int (raid_device, "md/degraded", NULL);
+          sync_action = udisks_linux_device_read_sysfs_attr (raid_device, "md/sync_action", NULL);
+          sync_completed = udisks_linux_device_read_sysfs_attr (raid_device, "md/sync_completed", NULL);
+          bitmap_location = udisks_linux_device_read_sysfs_attr (raid_device, "md/bitmap/location", NULL);
         }
 
       if (mdraid_has_stripes (level))
         {
-          chunk_size = read_sysfs_attr_as_uint64 (raid_device->udev_device, "md/chunk_size");
+          chunk_size = udisks_linux_device_read_sysfs_attr_as_uint64 (raid_device, "md/chunk_size", NULL);
         }
     }
   udisks_mdraid_set_degraded (iface, degraded);
@@ -357,7 +341,7 @@ udisks_linux_mdraid_update (UDisksLinuxMDRaid       *mdraid,
         }
 
       /* this is KiB/s (see drivers/md/md.c:sync_speed_show() */
-      sync_rate = read_sysfs_attr_as_uint64 (raid_device->udev_device, "md/sync_speed") * 1024;
+      sync_rate = udisks_linux_device_read_sysfs_attr_as_uint64 (raid_device, "md/sync_speed", NULL) * 1024;
       if (sync_rate > 0)
         {
           guint64 num_bytes_remaining = (num_sectors - completed_sectors) * 512ULL;
@@ -470,20 +454,19 @@ udisks_linux_mdraid_update (UDisksLinuxMDRaid       *mdraid,
                 }
 
               snprintf (buf, sizeof (buf), "md/%s/state", file_name);
-              member_state = read_sysfs_attr (raid_device->udev_device, buf);
+              member_state = udisks_linux_device_read_sysfs_attr (raid_device, buf, NULL);
               if (member_state != NULL)
                 {
                   member_state_elements = g_strsplit (member_state, ",", 0);
                 }
               else
                 {
-                  // g_variant_new() doesn't like NULL pointers for empty
-                  // arrays
+                  /* g_variant_new() doesn't like NULL pointers for empty arrays */
                   member_state_elements = g_new0 (gchar*, 1);
                 }
 
               snprintf (buf, sizeof (buf), "md/%s/slot", file_name);
-              member_slot = read_sysfs_attr (raid_device->udev_device, buf);
+              member_slot = udisks_linux_device_read_sysfs_attr (raid_device, buf, NULL);
               if (member_slot != NULL)
                 {
                   if (g_strcmp0 (member_slot, "none") != 0)
@@ -491,7 +474,7 @@ udisks_linux_mdraid_update (UDisksLinuxMDRaid       *mdraid,
                 }
 
               snprintf (buf, sizeof (buf), "md/%s/errors", file_name);
-              member_errors = read_sysfs_attr_as_uint64 (raid_device->udev_device, buf);
+              member_errors = udisks_linux_device_read_sysfs_attr_as_uint64 (raid_device, buf, NULL);
 
               g_ptr_array_add (p,
                                g_variant_new ("(oi^asta{sv})",
@@ -529,6 +512,7 @@ udisks_linux_mdraid_update (UDisksLinuxMDRaid       *mdraid,
                                                                                 uuid));
 
  out:
+  g_dbus_interface_skeleton_flush (G_DBUS_INTERFACE_SKELETON (mdraid));
   if (raid_data)
       bd_md_examine_data_free (raid_data);
   g_free (sync_completed);
@@ -681,7 +665,7 @@ handle_start (UDisksMDRaid           *_mdraid,
                                                      wait_for_md_block_object,
                                                      object,
                                                      NULL,
-                                                     10, /* timeout_seconds */
+                                                     UDISKS_DEFAULT_WAIT_TIMEOUT,
                                                      &error);
   if (block_object == NULL)
     {
@@ -1076,7 +1060,7 @@ handle_remove_device (UDisksMDRaid           *_mdraid,
 
   if (opt_wipe)
     {
-      if (!bd_fs_wipe (member_device_file, TRUE, &error))
+      if (!bd_fs_wipe (member_device_file, TRUE, TRUE, &error))
         {
           g_prefix_error (&error,
                           "Error wiping '%s' after removal from RAID array '%s': ",
