@@ -425,64 +425,6 @@ udisks_state_check (UDisksState *state)
                          state);
 }
 
-
-typedef struct
-{
-  UDisksState *state;
-  gboolean     finished;
-  GCond        cond;
-  GMutex       data_mutex;
-} UDisksStateCheckSyncData;
-
-static gboolean
-udisks_state_check_sync_func (UDisksStateCheckSyncData *data)
-{
-  udisks_state_check_in_thread (data->state);
-
-  /* signal the calling thread the cleanup has finished */
-  g_mutex_lock (&data->data_mutex);
-  data->finished = TRUE;
-  g_cond_signal (&data->cond);
-  g_mutex_unlock (&data->data_mutex);
-
-  return FALSE;
-}
-
-/**
- * udisks_state_check_sync:
- * @state: A #UDisksState.
- *
- * Causes the clean-up thread for @state to check if anything should be cleaned up and perform the cleanup.
- *
- * This can be called from any thread and in contrast to udisks_state_check() will block the calling thread until cleanup is finished.
- */
-void
-udisks_state_check_sync (UDisksState *state)
-{
-  UDisksStateCheckSyncData data = {0, };
-
-  g_return_if_fail (UDISKS_IS_STATE (state));
-  g_return_if_fail (state->thread != NULL);
-
-  g_cond_init (&data.cond);
-  g_mutex_init (&data.data_mutex);
-  data.state = state;
-  data.finished = FALSE;
-
-  g_mutex_lock (&data.data_mutex);
-  g_main_context_invoke (state->context,
-                         (GSourceFunc) udisks_state_check_sync_func,
-                         &data);
-
-  /* wait for the mainloop running in the cleanup thread to process our injected task */
-  while (!data.finished)
-    g_cond_wait (&data.cond, &data.data_mutex);
-  g_mutex_unlock (&data.data_mutex);
-
-  g_cond_clear (&data.cond);
-  g_mutex_clear (&data.data_mutex);
-}
-
 /**
  * udisks_state_check_block:
  * @state: A #UDisksState.
@@ -627,34 +569,6 @@ lookup_asv (GVariant    *asv,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-static void
-trigger_change_uevent (const gchar *sysfs_path)
-{
-  gchar* path = NULL;
-  gint fd = -1;
-
-  g_return_if_fail (sysfs_path != NULL);
-
-  path = g_strconcat (sysfs_path, "/uevent", NULL);
-  fd = open (path, O_WRONLY);
-  if (fd < 0)
-    {
-      udisks_warning ("Error opening %s for trigerring change uevent: %m", path);
-      goto out;
-    }
-
-  if (write (fd, "change", sizeof "change" - 1) != sizeof "change" - 1)
-    {
-      udisks_warning ("Error writing 'change' to file %s: %m", path);
-      goto out;
-    }
-
- out:
-  if (fd >= 0)
-    close (fd);
-  g_free (path);
-}
-
 /* returns TRUE if the entry should be kept */
 static gboolean
 udisks_state_check_mounted_fs_entry (UDisksState  *state,
@@ -727,7 +641,7 @@ udisks_state_check_mounted_fs_entry (UDisksState  *state,
           gchar *device_file;
 
           device_file = udisks_linux_block_object_get_device_file (UDISKS_LINUX_BLOCK_OBJECT (block_object));
-          udisks_notice ("udisks_state_check_mounted_fs_entry: block device %s is busy, skipping cleanup", device_file);
+          udisks_debug ("udisks_state_check_mounted_fs_entry: block device %s is busy, skipping cleanup", device_file);
           g_free (device_file);
 
           keep = TRUE;
@@ -738,7 +652,7 @@ udisks_state_check_mounted_fs_entry (UDisksState  *state,
 
   if (realpath (mount_point_str, mount_point) == NULL)
     {
-      udisks_critical ("udisks_state_check_mounted_fs_entry: mountpoint %s is invalid, cannot recover the canonical path ", mount_point_str);
+      udisks_critical ("udisks_state_check_mounted_fs_entry: mountpoint %s is invalid, cannot recover the canonical path: %m", mount_point_str);
     }
 
   fstab_mount_value = lookup_asv (details, "fstab-mount");
@@ -894,7 +808,7 @@ udisks_state_check_mounted_fs_entry (UDisksState  *state,
            */
           if (change_sysfs_path != NULL)
             {
-              trigger_change_uevent (change_sysfs_path);
+              udisks_daemon_util_trigger_uevent (state->daemon, NULL, change_sysfs_path);
             }
         }
 
@@ -2441,7 +2355,7 @@ udisks_state_get (UDisksState           *state,
   g_warn_if_fail (ret != NULL);
   g_variant_ref_sink (ret);
 
-  contents = NULL; /* ownership transfered to the returned GVariant */
+  contents = NULL; /* ownership transferred to the returned GVariant */
 
  out:
   g_free (contents);
